#version 330 core

layout(location = 0) in vec3 a_Position;
layout(location = 1) in vec3 a_Normal;
layout(location = 2) in vec2 a_UV;

uniform mat4 u_ProjectionMatrix;
uniform mat4 u_ViewMatrix;
uniform mat4 u_ModelMatrix;
uniform mat3 u_NormalMatrix;
uniform mat4 u_LightViewProjectionMatrix;

out VertexShader {
	vec3 fragpos_worldspace;
	vec3 normal_worldspace;
	vec2 uv;
	vec4 fragpos_lightspace;
} vs_out;

void main()
{
	// Set position vector
	vs_out.fragpos_worldspace = (u_ModelMatrix * vec4(a_Position, 1.0)).xyz;
	
	// Set normal vector
	vs_out.normal_worldspace = u_NormalMatrix * a_Normal;

	// Set uv coordinates
	vs_out.uv = a_UV;

	// Set position vector (transformed into light space for shadow mapping)
	vs_out.fragpos_lightspace = u_LightViewProjectionMatrix * vec4(vs_out.fragpos_worldspace, 1.0);

	// Calculate vertex position based on mvp matrix
	gl_Position = u_ProjectionMatrix * u_ViewMatrix * u_ModelMatrix * vec4(a_Position, 1.0);
}