#version 430 core

in VertexShader {
	vec2 uv;
} vs_in;

out vec4 pixelColor;

uniform sampler2D u_ScreenTexture;
uniform sampler2D u_BloomTexture;

uniform bool u_Inversion;
uniform bool u_HDR;
uniform bool u_Bloom;
uniform bool u_GaussianBlur;
uniform float u_Exposure;

void main()
{
    vec3 color;

    // Average gamma of most displays
	const float gamma = 2.2;
    
    // The color from the HDR buffer
    vec3 hdrColor = texture(u_ScreenTexture, vs_in.uv).rgb;
    
    // The color from the Bloom buffer
    vec3 bloomColor = texture(u_BloomTexture, vs_in.uv).rgb;


    if (u_Bloom)
    {
        // Additive blending
        hdrColor += bloomColor;
    }


    if (u_HDR) {
        vec3 result;
        // Exposure tone mapping (default at 1.0)
        result = vec3(1.0) - exp(-hdrColor * u_Exposure);

		// Gamma correction (tone mapped color from buffer is raised to the power of the gamma correction)
        result = pow(result, vec3(1.0 / gamma));
        color = result;
    } 
    else {
        color = hdrColor;
    }

    if (u_Inversion) {
        color = vec3(1.0 - color);
    }

    // Final post processing result
    pixelColor = vec4(color, 1.0);

    if (u_GaussianBlur) {
        pixelColor = vec4(bloomColor, 1.0);
    }
}